/* Audio Library for Teensy 3.X
   Copyright (c) 2014, Pete (El Supremo)

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
   THE SOFTWARE.
*/

#include "effect_phase.h"
#include "arm_math.h"


boolean AudioEffectPhasor::begin(short *delayline, int d_length, int delay_offset, int d_depth, float delay_rate)
{
  boolean all_ok = true;

  if (0) {
    Serial.print("AudioEffectPhasor.begin(offset = ");
    Serial.print(delay_offset);
    Serial.print(", depth = ");
    Serial.print(d_depth);
    Serial.print(", rate = ");
    Serial.print(delay_rate, 3);
    Serial.println(")");
    Serial.print("    FLANGE_DELAY_LENGTH = ");
    Serial.println(d_length);
  }
  delay_length = d_length / 2;
  l_delayline = delayline;

  delay_depth = d_depth;
  // initial index
  l_delay_rate_index = 0;
  l_circ_idx = 0;
  delay_rate_incr = (delay_rate * 2147483648.0) / AUDIO_SAMPLE_RATE_EXACT;
  //Serial.println(delay_rate_incr,HEX);

  delay_offset_idx = delay_offset;
  // Allow the passthru code to go through
  if (delay_offset_idx < -1) {
    delay_offset_idx = 0;
    all_ok = false;
  }
  if (delay_offset_idx >= delay_length) {
    delay_offset_idx = delay_length - 1;
    all_ok = false;
  }
  return (all_ok);
}


boolean AudioEffectPhasor::voices(int fb, int d_depth, float delay_rate)
{
  boolean all_ok = true;

  depth = d_depth;
  rate = delay_rate;
  feedback = fb;

  delay_rate_incr = (delay_rate * 2147483648.0) / AUDIO_SAMPLE_RATE_EXACT;

  return (all_ok);
}

void AudioEffectPhasor::update(void)
{
  audio_block_t *block;
  short *bp;
  //calculate and update phaser sweep lfo...
  float d  = _dmin + (_dmax - _dmin) * ((sin( _lfoPhase ) + 1.f) / 2.f);
  float y;
  _lfoPhase += rate;
  if ( _lfoPhase >= M_PI * 2.f )
    _lfoPhase -= M_PI * 2.f;

  //update filter coeffs
  for ( int i = 0; i < 6; i++ )
    _alps[i].Delay( d );


  block = receiveWritable(0);
  if (block) {
    bp = block->data;
    // fill the delay line
    for (int i = 0; i < AUDIO_BLOCK_SAMPLES; i++) {
      //calculate output
      bp = block->data;
      y = _alps[0].Update(_alps[1].Update(_alps[2].Update(_alps[3].Update(_alps[4].Update(
                                            _alps[5].Update( *bp + _zm1 * feedback ))))));
      _zm1 = y;

      *bp = *bp + y * depth;
      bp++;
    }
    transmit(block, 0);
  }
  release(block);
}




