/* Audio Library for Teensy 3.X
   Copyright (c) 2014, Pete (El Supremo)

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
   THE SOFTWARE.
*/

#ifndef effect_phasor_h_
#define effect_phasor_h_

#include "Arduino.h"
#include "AudioStream.h"


class AudioEffectPhasor :
  public AudioStream
{
  public:
    AudioEffectPhasor(void):AudioStream(1, inputQueueArray),
      _lfoPhase( 0.f ), depth( 1.f ), feedback( .7f ), _zm1( 0.f )
     {
      Range( 440.f, 1600.f );
      Rate( .5f );
    }

    boolean begin(short *delayline, int d_length, int delay_offset, int d_depth, float delay_rate);
    boolean voices(int delay_offset, int d_depth, float delay_rate);
    virtual void update(void);

    void Range( float fMin, float fMax ) { // Hz
      _dmin = fMin / (AUDIO_SAMPLE_RATE_EXACT / 2.f);
      _dmax = fMax / (AUDIO_SAMPLE_RATE_EXACT / 2.f);
    }

    float Rate( float rate ) { // cps
      _lfoInc = 2.f * M_PI * (rate / AUDIO_SAMPLE_RATE_EXACT);
      return _lfoInc * 1000.f;
    }

  private:
    audio_block_t *inputQueueArray[1];
    short *l_delayline;
    int delay_length;
    short l_circ_idx;
    int delay_depth;
    int delay_offset_idx;
    int   delay_rate_incr;
    unsigned int l_delay_rate_index;

    class AllpassDelay {

      public:
        AllpassDelay()
          : _a1( 0.f )
          , _zm1( 0.f )
        {}

        void Delay( float delay ) { //sample delay time
          _a1 = (1.f - delay) / (1.f + delay);
        }

        float Update( float inSamp ) {
          float y = inSamp * -_a1 + _zm1;
          _zm1 = y * _a1 + inSamp;

          return y;
        }
      private:
        float _a1, _zm1;
    };

    AllpassDelay _alps[6];

    float _dmin, _dmax; //range
    float _lfoPhase;
    float _lfoInc;
    float depth, rate, feedback;

    float _zm1;

};

#endif
