/*
 * Waveshaper for Teensy 3.X audio
 *
 * Copyright (c) 2017 Damien Clarke, http://damienclarke.me
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <Arduino.h>
#include "effect_waveshaper_f32.h"

AudioEffectWaveshaper_F32::~AudioEffectWaveshaper_F32()
{
}

void AudioEffectWaveshaper_F32::shape(float* waveshape, int length)
{
  // length must be bigger than 1 and equal to a power of two + 1
  // anything else means we don't continue
  if(!waveshape || length < 2 || length > 32769 || ((length - 1) & (length - 2))) return;
	this->waveshape =waveshape; 
  int index = length - 1;
  l2=index/2;
}

void AudioEffectWaveshaper_F32::update(void)
{
  if(!waveshape) return;

  audio_block_f32_t *block;
  block = receiveWritable_f32();
  if (!block) return;

  float x, xa;
  float ya, yb;
  for (int i = 0; i < AUDIO_BLOCK_SAMPLES; i++) {
    // bring int16_t data into uint16_t range
	x=block->data[i];
    xa = (1+ block->data[i])*l2;
    ya = waveshape[int(xa)];
    yb = waveshape[int(xa + 1)];
    block->data[i] = ya + (yb - ya)*(xa-int(xa)) ;
  }

  AudioStream_F32::transmit(block);
  AudioStream_F32::release(block);
}
